#!/usr/bin/python
#
# Copyright (C) 2006 Jesus Espino Garcia <jespino@di.uc3m.es>
#
# Based on metadistros project of Juan Jesus Ojeda <juanje@interactors.coop>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
import gtk
import gtk.glade
import gobject
import os
import shutil
import re
import sys
import vte

def gtk_main_quit(widget):
	gtk.main_quit()

def get_dist_config(dist,parameter):
	config_file = file("dists/"+dist+"/dist.config","r")
	value = []
	for line in config_file:
		pair = re.match("(^.*)=[\"\'](.*)[\"\']",line)
		if pair:
			if pair.group(1)==parameter:
				value.append(pair.group(2))
	return value

def get_gendist_config(parameter):
	config_file = file("gendist.config","r")
	value = ""
	for line in config_file:
		pair = re.match("(^.*)=[\"\'](.*)[\"\']",line)
		if pair:
			if pair.group(1)==parameter:
				value = pair.group(2)
	return value

def write_gendist_config():
	iso_file_input = xml.get_widget("iso_file_input")
	iso_file = iso_file_input.get_text()
	boot_type_combo = xml.get_widget("boot_type_combo")
	boot_type = boot_type_combo.get_active_text()
	isolinux_input = xml.get_widget("isolinux_input")
	isolinux = isolinux_input.get_text()
	grub_input = xml.get_widget("grub_input")
	grub = grub_input.get_text()
	gfxboot_input = xml.get_widget("gfxboot_input")
	gfxboot = gfxboot_input.get_text()
	default_dist_combo = xml.get_widget("default_dist_combo")
	default_dist = default_dist_combo.get_active_text()

	gendist_config_file = file("gendist.config","w")
	gendist_config_file.write("# CONFIGURATION\n")
	gendist_config_file.write("ISO_NAME=\""+iso_file+"\"\n")
	gendist_config_file.write("\n")
	gendist_config_file.write("# Boot\n")
	gendist_config_file.write("BOOT_TYPE=\""+boot_type+"\" # Method (isolinux/grub/gfxboot)\n")
	gendist_config_file.write("ISOLINUXLOGO=\""+isolinux+"\" # isolinux logo\n")
	gendist_config_file.write("GRUBLOGO=\""+grub+"\" # grub logo\n")
	gendist_config_file.write("GFXBOOT_THEME=\""+gfxboot+"\" # gfxboot Theme\n")
	gendist_config_file.write("\n")
	gendist_config_file.write("# DEFAULT DISTRIBUTION (if \"\" default\n")
	gendist_config_file.write("DEFAULT_DIST=\""+default_dist+"\"\n")
	gendist_config_file.close()

def write_dist_config(dist_name):
	dist_version_input = xml.get_widget("dist_version_input")
	dist_version = dist_version_input.get_text()

	deb_version_input = xml.get_widget("deb_version_input")
	deb_version = deb_version_input.get_text()

	base_mirror_input = xml.get_widget("base_mirror_input")
	base_mirror = base_mirror_input.get_text()

	hostname_input = xml.get_widget("hostname_input")
	hostname = hostname_input.get_text()

	dns_servers_combo = xml.get_widget("dns_servers_combo")
	dns_servers_model = dns_servers_combo.get_model()
	dns_servers = []
	for dns_server_model in dns_servers_model:
		dns_servers.append(dns_servers_model.get_value(dns_server_model.iter,0))

	splash_logo_input = xml.get_widget("splash_logo_input")
	splash_logo = splash_logo_input.get_text()

	doc_directory_input = xml.get_widget("doc_directory_input")
	doc_directory = doc_directory_input.get_text()

	kernel_version_combo = xml.get_widget("kernel_version_combo")
	kernel_version = kernel_version_combo.get_active_text()

	repositories_combo = xml.get_widget("repositories_combo")
	repositories_model = repositories_combo.get_model()
	repositories = []
	for repository_model in repositories_model:
		repositories.append(repositories_model.get_value(repository_model.iter,0))

	packages_combo = xml.get_widget("packages_combo")
	packages_model = packages_combo.get_model()
	packages = []
	for package_model in packages_model:
		packages.append(packages_model.get_value(package_model.iter,0))

	modules_treeview = xml.get_widget("modules_treeview")
	modules_model = modules_treeview.get_model()
	modules = []
	for module_model in modules_model:
		if modules_model.get_value(module_model.iter,2) == "#00BC57":
			mod_string = "MODS_AND_PARMS="+modules_model.get_value(module_model.iter,0)
			mod_string += ","+modules_model.get_value(module_model.iter,1)
			modules.append(mod_string)
		elif modules_model.get_value(module_model.iter,2) == "#DC0300":
			mod_string = "#MODS_AND_PARMS="+modules_model.get_value(module_model.iter,0)
			mod_string += ","+modules_model.get_value(module_model.iter,1)
			modules.append(mod_string)

	gendist_config_file = file("dists/"+dist_name+"/dist.config","w")
	gendist_config_file.write("# DISTRIBUTION CONFIG\n")
	gendist_config_file.write("VERSION=\""+dist_version+"\"\n")
	gendist_config_file.write("\n")
	gendist_config_file.write("# IMAGE CONFIG\n")
	gendist_config_file.write("DEB_VERSION=\""+deb_version+"\"\n")
	gendist_config_file.write("BASE_MIRROR=\""+base_mirror+"\"\n")
	for repository in repositories:
		gendist_config_file.write("REPOSITORIES=\""+repository+"\"\n")
	for dns_server in dns_servers:
		gendist_config_file.write("DNS_SERVER=\""+dns_server+"\"\n")
	gendist_config_file.write("HOSTNAME=\""+hostname+"\"\n")
	for package in packages:
		gendist_config_file.write("PACKAGES=\""+package+"\"\n")
	gendist_config_file.write("\n")
	gendist_config_file.write("# INITRAMFS CONFIG\n")
	gendist_config_file.write("KERNEL_VERSION=\""+kernel_version+"\"\n")
	gendist_config_file.write("SPLASH_LOGO=\"/usr/share/gendist/artwork/gendist-splash.png\" # usplash logo\n")
	gendist_config_file.write("\n")
	gendist_config_file.write("# INITRAMFS MODULES (Comment mods for no use it)\n")
	for module in modules:
		gendist_config_file.write(module+"\"\n")
	gendist_config_file.write("\n")
	gendist_config_file.write("# DOCUMENTATION CONFIG\n")
	gendist_config_file.write("DOC_DIRECTORY=\""+doc_directory+"\"\n")
	gendist_config_file.close()

def load_gendist_config():
	"""Fill the fields of gendist config form."""
	iso_file_input = xml.get_widget("iso_file_input")
	boot_type_combo = xml.get_widget("boot_type_combo")
	isolinux_input = xml.get_widget("isolinux_input")
	grub_input = xml.get_widget("grub_input")
	gfxboot_input = xml.get_widget("gfxboot_input")
	dist_name_combo = xml.get_widget("dist_name_combo")
	add_dist_button = xml.get_widget("add_dist_button")
	edit_dist_button = xml.get_widget("edit_dist_button")
	del_dist_button = xml.get_widget("del_dist_button")
	execute_frame = xml.get_widget("execute_frame")
	preferences_frame = xml.get_widget("preferences_frame")

	list_dists_combo1 = xml.get_widget("list_dists_combo1")
	list_dists_combo2 = xml.get_widget("list_dists_combo2")
	list_dists_combo3 = xml.get_widget("list_dists_combo3")
	list_dists_combo4 = xml.get_widget("list_dists_combo4")

	execute_frame.show()
	preferences_frame.show()

	add_dist_button.set_sensitive(False)
	edit_dist_button.set_sensitive(False)
	del_dist_button.set_sensitive(False)

	iso_file_input.set_text(get_gendist_config("ISO_NAME"))
	isolinux_input.set_text(get_gendist_config("ISOLINUXLOGO"))
	grub_input.set_text(get_gendist_config("GRUBLOGO"))
	gfxboot_input.set_text(get_gendist_config("GFXBOOT_THEME"))
	if get_gendist_config("BOOT_TYPE") == "gfxboot":
		boot_type_combo.set_active(0)
	elif get_gendist_config("BOOT_TYPE") == "grub":
		boot_type_combo.set_active(1)
	elif get_gendist_config("BOOT_TYPE") == "isolinux":
		boot_type_combo.set_active(2)

	default_dist = get_gendist_config("DEFAULT_DIST")
	if default_dist == "":
		if os.path.exists("dists"):
			default_dist = os.listdir("dists")[0]
	default_dist_combo = xml.get_widget("default_dist_combo")
	renderer = gtk.CellRendererText()
	default_dist_combo.pack_start(renderer, True)
	default_dist_combo.add_attribute(renderer, 'text', 0)

	counter = 0
	default_dist_index = 0
	dists_model      = gtk.ListStore( gobject.TYPE_STRING )
	if os.path.exists("dists"):
		directories = os.listdir("dists")
		directories.sort()
		for dist in directories:
			dists_iter = dists_model.insert_before(None)
			dists_model.set(dists_iter,0,dist)
			if dist == default_dist:
				default_dist_index = counter
			counter+=1
	default_dist_combo.set_model( dists_model )
	default_dist_combo.set_active(default_dist_index)

	dist_name_combo.set_model(dists_model)
	dist_name_combo.set_text_column(0)

	list_dists_combo1.set_model(dists_model)
	list_dists_combo2.set_model(dists_model)
	list_dists_combo3.set_model(dists_model)
	list_dists_combo4.set_model(dists_model)

def load_dist_config(value):
	dist_version_input = xml.get_widget("dist_version_input")
	deb_version_input = xml.get_widget("deb_version_input")
	base_mirror_input = xml.get_widget("base_mirror_input")
	hostname_input = xml.get_widget("hostname_input")
	dns_servers_combo = xml.get_widget("dns_servers_combo")
	splash_logo_input = xml.get_widget("splash_logo_input")
	doc_directory_input = xml.get_widget("doc_directory_input")
	kernel_version_combo = xml.get_widget("kernel_version_combo")
	repositories_combo = xml.get_widget("repositories_combo")
	packages_combo = xml.get_widget("packages_combo")
	modules_treeview = xml.get_widget("modules_treeview")

	generate_dist_button = xml.get_widget("generate_dist_button")
	config_dist_button = xml.get_widget("config_dist_button")
	mk_initramfs_button = xml.get_widget("mk_initramfs_button")
	build_initramfs_button = xml.get_widget("build_initramfs_button")
	base_button = xml.get_widget("base_button")
	install_pkg_button = xml.get_widget("install_pkg_button")
	install_deb_button = xml.get_widget("install_deb_button")
	clean_button = xml.get_widget("clean_button")
	squashfs_button = xml.get_widget("squashfs_button")
	gen_iso_button2 = xml.get_widget("gen_iso_button2")

	if os.path.exists("master/boot/grub/menu.lst") or os.path.exists("master/boot/isolinux/isolinux.bin"):
		gen_iso_button2.set_sensitive(True)
	else:
		gen_iso_button2.set_sensitive(False)

	generate_dist_button.set_sensitive(False)
	config_dist_button.set_sensitive(False)
	mk_initramfs_button.set_sensitive(False)
	build_initramfs_button.set_sensitive(False)
	base_button.set_sensitive(False)
	install_pkg_button.set_sensitive(False)
	install_deb_button.set_sensitive(False)
	clean_button.set_sensitive(False)
	squashfs_button.set_sensitive(False)

	if get_dist_config(value,"VERSION")[0] > 0:
		dist_version = get_dist_config(value,"VERSION")[0]
		dist_version_input.set_text(dist_version)
	else:
		dist_version=""

	if get_dist_config(value,"DEB_VERSION")[0] > 0:
		deb_version = get_dist_config(value,"DEB_VERSION")[0]
		deb_version_input.set_text(deb_version)
	else:
		deb_version=""

	if get_dist_config(value,"BASE_MIRROR")[0] > 0:
		base_mirror = get_dist_config(value,"BASE_MIRROR")[0]
		base_mirror_input.set_text(base_mirror)
	else:
		base_mirror = ""

	if get_dist_config(value,"HOSTNAME")[0] > 0:
		hostname = get_dist_config(value,"HOSTNAME")[0]
		hostname_input.set_text(hostname)
	else:
		hostname = ""

	if get_dist_config(value,"SPLASH_LOGO")[0] > 0:
		splash_logo = get_dist_config(value,"SPLASH_LOGO")[0]
		splash_logo_input.set_text(splash_logo)
	else:
		splash_logo = ""

	if get_dist_config(value,"DOC_DIRECTORY")[0] > 0:
		doc_directory = get_dist_config(value,"DOC_DIRECTORY")[0]
		doc_directory_input.set_text(doc_directory)
	else:
		doc_directory = ""

	if get_dist_config(value,"KERNEL_VERSION")[0] > 0:
		kernel_version = get_dist_config(value,"KERNEL_VERSION")[0]
	else:
		kernel_version = ""

	if kernel_version == "":
		kernel_version = os.uname()[2]

	if kernel_version_combo.get_model() == None:
		kernel_version_combo.set_model(kernels_model)

	for kernel_model in kernels_model:
		if kernels_model.get_value(kernel_model.iter,0) == kernel_version:
			kernel_version_combo.set_active_iter(kernel_model.iter)
	
	# Repositories model
	repositories = get_dist_config(value,"REPOSITORIES")
	while repositories.count("") > 0:
		repositories.remove("")

	repositories_model = gtk.ListStore(gobject.TYPE_STRING)
	for repository in repositories:
		repositories_iter = repositories_model.insert_before(None)
		repositories_model.set_value(repositories_iter,0,repository)
	repositories_combo.set_model(repositories_model)
	repositories_combo.set_text_column(0)
	changed_repositories_combo(repositories_combo)

	# DNS servers model
	dns_servers = get_dist_config(value,"DNS_SERVER")
	while dns_servers.count("") > 0:
		dns_servers.remove("")

	dns_servers_model = gtk.ListStore(gobject.TYPE_STRING)
	for dns_server in dns_servers:
		dns_servers_iter = dns_servers_model.insert_before(None)
		dns_servers_model.set_value(dns_servers_iter,0,dns_server)
	dns_servers_combo.set_model(dns_servers_model)
	dns_servers_combo.set_text_column(0)
	changed_dns_servers_combo(dns_servers_combo)

	# Packages model 
	packages = get_dist_config(value,"PACKAGES")
	while packages.count("") > 0:
		packages.remove("")

	packages_model = gtk.ListStore(gobject.TYPE_STRING)
	for package in packages:
		packages_iter = packages_model.insert_before(None)
		packages_model.set_value(packages_iter,0,package)
	packages_combo.set_model(packages_model)
	packages_combo.set_text_column(0)
	changed_packages_combo(packages_combo)


	# Modules model
	modules = get_dist_config(value,"MODS_AND_PARMS")
	disabled_modules = get_dist_config(value,"#MODS_AND_PARMS")

	while modules.count("") > 0:
		modules.remove("")

	model = modules_treeview.get_model()
	for line_module in model:
		iter = line_module.iter
		path = line_module.path
		for module in modules:
			mod_name=module.split(",")[0]
			mod_parms=module.replace(mod_name+",","")
			if mod_name == mod_parms:
				mod_parms = ""
			if model.get_value(iter,0) == mod_name:
				model.set_value(iter,1,mod_parms)
				model.set_value(iter,2,"#00BC57")

		for module in disabled_modules:
			mod_name=module.split(",")[0]
			mod_parms=module.replace(mod_name+",","")
			if mod_name == mod_parms:
				mod_parms = ""
			if model.get_value(iter,0) == mod_name:
				model.set_value(iter,1,mod_parms)
				model.set_value(iter,2,"#DC0300")

def clicked_disconnect_module(widget):
	modules_treeview = xml.get_widget("modules_treeview")
	model,iter = modules_treeview.get_selection().get_selected()
	model.set_value(iter,2,"#DC0300")

def clicked_connect_module(widget):
	modules_treeview = xml.get_widget("modules_treeview")
	model,iter = modules_treeview.get_selection().get_selected()
	model.set_value(iter,2,"#00BC57")

def child_exited_vte(widget):
	back_button = xml.get_widget("back_button")
	back_button.set_sensitive(True)
	statusbar = xml.get_widget("statusbar")
	statusbar.pop(0)
	statusbar.push(0,"Finished")

def clicked_back_button(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_current_page(0)
	changed_list_dists_combo1(xml.get_widget("list_dists_combo1"))
	changed_list_dists_combo2(xml.get_widget("list_dists_combo2"))
	changed_list_dists_combo3(xml.get_widget("list_dists_combo3"))
	changed_list_dists_combo4(xml.get_widget("list_dists_combo4"))

def exec_vte_command(command,args):
	back_button = xml.get_widget("back_button")
	back_button.set_sensitive(False)
	terminal.fork_command(command,args)
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_current_page(3)

def clicked_mk_initramfs(widget):
	list_dists_combo2 = xml.get_widget("list_dists_combo2")
	statusbar = xml.get_widget("statusbar")
	distribution = list_dists_combo2.get_active_text()
	exec_vte_command("gendist",["gendist",distribution,"mk_initramfs"])
	statusbar.push(0,"Generating "+distribution+"'s base initramfs...")

def clicked_build_initramfs(widget):
	list_dists_combo2 = xml.get_widget("list_dists_combo2")
	distribution = list_dists_combo2.get_active_text()
	exec_vte_command("gendist",["gendist",distribution,"initramfs"])
	statusbar = xml.get_widget("statusbar")
	statusbar.push(0,"Building "+distribution+"'s initramfs file...")

def clicked_base_button(widget):
	list_dists_combo3 = xml.get_widget("list_dists_combo3")
	distribution = list_dists_combo3.get_active_text()
	exec_vte_command("gendist",["gendist",distribution,"source"])
	statusbar = xml.get_widget("statusbar")
	statusbar.push(0,"Generating "+distribution+"'s base system...")

def clicked_clean_button(widget):
	list_dists_combo3 = xml.get_widget("list_dists_combo3")
	distribution = list_dists_combo3.get_active_text()
	exec_vte_command("gendist",["gendist",distribution,"clean"])
	statusbar = xml.get_widget("statusbar")
	statusbar.push(0,"Cleaning "+distribution+"'s packages cache...")

def clicked_squashfs_button(widget):
	list_dists_combo3 = xml.get_widget("list_dists_combo3")
	distribution = list_dists_combo3.get_active_text()
	exec_vte_command("gendist",["gendist",distribution,"squashfs"])
	statusbar = xml.get_widget("statusbar")
	statusbar.push(0,"Building "+distribution+"'s squashfs file...")

def clicked_gen_iso_button(widget):
	exec_vte_command("gendist",["gendist","iso"])
	statusbar = xml.get_widget("statusbar")
	statusbar.push(0,"Generating iso file...")

def clicked_gen_dists_button(widget):
	list_dists_combo1 = xml.get_widget("list_dists_combo1")
	distribution = list_dists_combo1.get_active_text()
	exec_vte_command("gendist",["gendist","generate_all"])
	statusbar = xml.get_widget("statusbar")
	statusbar.push(0,"Generating all distributions...")

def clicked_gen_dist_button(widget):
	list_dists_combo1 = xml.get_widget("list_dists_combo1")
	distribution = list_dists_combo1.get_active_text()
	exec_vte_command("gendist",["gendist",distribution,"generate"])
	statusbar = xml.get_widget("statusbar")
	statusbar.push(0,"Generating "+distribution+" distribution...")

def clicked_make_boot_button(widget):
	exec_vte_command("gendist",["gendist","mk_boot"])
	statusbar = xml.get_widget("statusbar")
	statusbar.push(0,"Generating boot system...")

def clicked_edit_module(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	modules_treeview = xml.get_widget("modules_treeview")
	model,iter = modules_treeview.get_selection().get_selected()
	module_name = model.get_value(iter,0)
	module_parms = model.get_value(iter,1).split(",")

	window = gtk.Window()
	window.connect("destroy",clicked_cancel_edit_module_window,window)
	frame = gtk.Frame()
	frame_vbox = gtk.VBox()
	hbox_buttons = gtk.HBox()

	modules_entries = []

	for parm in get_module_parms(module_name):
		hbox = gtk.HBox()
		if parm[0] == "STRING":
			label = gtk.Label()
			label.set_text(parm[1])
			entry = gtk.Entry()
			modules_entries.append([module_name,parm[0],parm[1],entry])
			for module in module_parms:
				if module.split("=")[0] == parm[1]:
					entry.set_text(module.split("=")[1])
			label.show()
			hbox.add(label)
		elif parm[0] == "IP":
			label = gtk.Label()
			label.set_text(parm[1])
			entry = gtk.Entry()
			modules_entries.append([module_name,parm[0],parm[1],entry])
			for module in module_parms:
				if module.split("=")[0] == parm[1]:
					entry.set_text(module.split("=")[1])
			label.show()
			hbox.add(label)
		elif parm[0] == "ENUM":
			label = gtk.Label()
			label.set_text(parm[1])

			params_model = gtk.ListStore( gobject.TYPE_STRING )
			renderer = gtk.CellRendererText()
			entry = gtk.ComboBox(params_model)
			entry.pack_start(renderer, True)
			entry.add_attribute(renderer, 'text', 0)

			params = parm[2]
			params.sort()
			for param in params:
				params_iter = params_model.insert_before(None)
				params_model.set(params_iter,0,param)
				for module in module_parms:
					if parm[1] == module.split("=")[0]:
						if module.split("=")[1] == param:
							entry.set_active_iter(params_iter)

			modules_entries.append([module_name,parm[0],parm[1],entry])

			label.show()
			hbox.add(label)
		elif parm[0] == "BOOL":
			entry = gtk.CheckButton(parm[1])
			modules_entries.append([module_name,parm[0],parm[1],entry])
			for module in module_parms:
				if module.split("=")[0] == parm[1]:
					if module.split("=")[1] == "yes":
						entry.set_active(True)
					elif module.split("=")[1] == "no":
						entry.set_active(False)
		entry.show()
		hbox.add(entry)
		hbox.show()
		frame_vbox.add(hbox)

	accept_button = gtk.Button(stock=gtk.STOCK_SAVE)
	accept_button.set_use_stock(True)
	accept_button.connect("clicked",clicked_accept_edit_module_window,modules_entries,window)
	cancel_button = gtk.Button(stock=gtk.STOCK_CANCEL)
	cancel_button.set_use_stock(True)
	cancel_button.connect("clicked",clicked_cancel_edit_module_window,window)
	hbox_buttons.add(accept_button)
	hbox_buttons.add(cancel_button)

	hbox_buttons.show()
	frame_vbox.add(hbox_buttons)
	frame_vbox.show()
	frame.add(frame_vbox)
	frame.show()
	window.add(frame)
	window.show_all()

def clicked_accept_edit_module_window(widget,modules_entries,window):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	modules_treeview = xml.get_widget("modules_treeview")
	model,iter = modules_treeview.get_selection().get_selected()
	parms_strings = []
	for parms in modules_entries:
		if parms[1] == "IP":
			parms_strings.append(parms[2]+"="+parms[3].get_text())
		elif parms[1] == "STRING":
			parms_strings.append(parms[2]+"="+parms[3].get_text())
		elif parms[1] == "BOOL":
			if parms[3].get_active():
				parms_strings.append(parms[2]+"=yes")
			else:
				parms_strings.append(parms[2]+"=no")
		elif parms[1] == "ENUM":
			parms_strings.append(parms[2]+"="+parms[3].get_active_text())
	model.set_value(iter,1,",".join(parms_strings))
	window.destroy()

def clicked_cancel_edit_module_window(widget,window):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	window.destroy()

def clicked_dist_edit(widget):
	main_notebook = xml.get_widget("main_notebook")
	dist_name_combo = xml.get_widget("dist_name_combo")
	load_dist_config(dist_name_combo.get_active_text())
	main_notebook.set_current_page(2)

def clicked_gendist_config(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_current_page(1)

def clicked_dist_config(widget):
	main_notebook = xml.get_widget("main_notebook")
	list_dists_combo4 = xml.get_widget("list_dists_combo4")
	load_dist_config(list_dists_combo4.get_active_text())
	main_notebook.set_current_page(2)

def clicked_project_new(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	project_new_folder = xml.get_widget("project_new_folder")
	project_new_folder.show()

def clicked_project_new_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	project_new_folder = xml.get_widget("project_new_folder")
	project_new_folder.hide()
	os.chdir(project_new_folder.get_filename())
	shutil.copy("/usr/share/gendist/gendist.config","gendist.config")
	load_gendist_config()

def clicked_project_new_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	create_folder_dialog = xml.get_widget("project_new_folder")
	create_folder_dialog.hide()

def clicked_project_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	project_open_folder = xml.get_widget("project_open_folder")
	project_open_folder.show()
	
def clicked_project_open_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	project_open_folder = xml.get_widget("project_open_folder")
	project_open_folder.hide()
	os.chdir(project_open_folder.get_filename())
	load_gendist_config()
	
def clicked_project_open_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	open_folder_dialog = xml.get_widget("project_open_folder")
	open_folder_dialog.hide()

def clicked_installdeb_button(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	installdeb_open_file = xml.get_widget("installdeb_open_file")
	installdeb_open_file.show()

def clicked_installdeb_open_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	list_dists_combo3 = xml.get_widget("list_dists_combo3")
	distribution = list_dists_combo3.get_active_text()
	installdeb_open_file = xml.get_widget("installdeb_open_file")
	exec_vte_command("gendist",["gendist",distribution,"installdeb"]+install_deb_open_file.get_filenames())
	installdeb_open_file.unselect_all()
	installdeb_open_file.hide()

def clicked_installdeb_open_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	installdeb_open_file = xml.get_widget("installdeb_open_file")
	installdeb_open_file.hide()

def clicked_install_pkg_button(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	install_package_dialog = xml.get_widget("install_package_dialog")
	install_package_dialog.show()

def clicked_install_package_ok(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	install_package_dialog = xml.get_widget("install_package_dialog")
	install_package_dialog.hide()
	simulate_checkbox = xml.get_widget("simulate_checkbox")
	install_package_entry = xml.get_widget("install_package_entry")
	list_dists_combo3 = xml.get_widget("list_dists_combo3")

	distribution = list_dists_combo3.get_active_text()

	if simulate_checkbox.get_active():
		exec_vte_command("gendist",["gendist",distribution,"sim_install",install_package_entry.get_text()])
	else:
		exec_vte_command("gendist",["gendist",distribution,"install",install_package_entry.get_text()])

	install_package_entry.set_text("")
	simulate_checkbox.set_active(False)

def clicked_install_package_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	install_package_dialog = xml.get_widget("install_package_dialog")
	install_package_dialog.hide()
	
def clicked_isolinux_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	isolinux_open = xml.get_widget("isolinux_open_file")
	isolinux_open.show()

def clicked_isolinux_open_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	isolinux_open = xml.get_widget("isolinux_open_file")
	isolinux_input = xml.get_widget("isolinux_input")
	isolinux_input.set_text(isolinux_open.get_filename())
	isolinux_open.hide()

def clicked_isolinux_open_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	isolinux_open = xml.get_widget("isolinux_open_file")
	isolinux_open.hide()

def clicked_grub_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	grub_open = xml.get_widget("grub_open_file")
	grub_open.show()

def clicked_grub_open_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	grub_open = xml.get_widget("grub_open_file")
	grub_input = xml.get_widget("grub_input")
	grub_input.set_text(grub_open.get_filename())
	grub_open.hide()

def clicked_grub_open_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	grub_open = xml.get_widget("grub_open_file")
	grub_open.hide()

def clicked_gfxboot_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	gfxboot_open = xml.get_widget("gfxboot_open_folder")
	gfxboot_open.show()

def clicked_gfxboot_open_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	gfxboot_open = xml.get_widget("gfxboot_open_folder")
	gfxboot_input = xml.get_widget("gfxboot_input")
	gfxboot_input.set_text(gfxboot_open.get_filename())
	gfxboot_open.hide()

def clicked_gfxboot_open_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	gfxboot_open = xml.get_widget("gfxboot_open_folder")
	gfxboot_open.hide()

def clicked_add_dist(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_current_page(2)
	dist_name_combo = xml.get_widget("dist_name_combo")
	dist_name = dist_name_combo.get_active_text()
	os.system("gendist add_dist "+dist_name)
	load_dist_config(dist_name)
	dists_model = dist_name_combo.get_model()
	dists_iter = dists_model.insert_before(None)
	dists_model.set(dists_iter,0,dist_name)


def changed_list_dists_combo1(widget):
	dist = widget.get_active_text()
	if dist == None:
		generate_dist_button = xml.get_widget("generate_dist_button")
		generate_dist_button.set_sensitive(False)
	else:
		generate_dist_button = xml.get_widget("generate_dist_button")
		generate_dist_button.set_sensitive(True)

def changed_list_dists_combo2(widget):
	mk_initramfs_button = xml.get_widget("mk_initramfs_button")
	build_initramfs_button = xml.get_widget("build_initramfs_button")
	gen_iso_button2 = xml.get_widget("gen_iso_button2")
	dist = widget.get_active_text()
	if dist == None:
		mk_initramfs_button.set_sensitive(False)
		build_initramfs_button.set_sensitive(False)
	else:
		mk_initramfs_button.set_sensitive(True)
		if os.path.exists("dists/"+dist+"/initramfs/scripts"):
			build_initramfs_button.set_sensitive(True)
		else:
			build_initramfs_button.set_sensitive(False)

		if os.path.exists("master/boot/grub/menu.lst") or os.path.exists("master/boot/isolinux/isolinux.bin"):
			gen_iso_button2.set_sensitive(True)
		else:
			gen_iso_button2.set_sensitive(False)

def changed_list_dists_combo3(widget):
	base_button = xml.get_widget("base_button")
	install_pkg_button = xml.get_widget("install_pkg_button")
	install_deb_button = xml.get_widget("install_deb_button")
	clean_button = xml.get_widget("clean_button")
	squashfs_button = xml.get_widget("squashfs_button")
	dist = widget.get_active_text()
	if dist == None:
		base_button.set_sensitive(False)
		install_pkg_button.set_sensitive(False)
		install_deb_button.set_sensitive(False)
		clean_button.set_sensitive(False)
		squashfs_button.set_sensitive(False)
	else:
		base_button.set_sensitive(True)
		if os.path.exists("dists/"+dist+"/image/bin"):
			install_pkg_button.set_sensitive(True)
			install_deb_button.set_sensitive(True)
			clean_button.set_sensitive(True)
			squashfs_button.set_sensitive(True)
		else:
			install_pkg_button.set_sensitive(False)
			install_deb_button.set_sensitive(False)
			clean_button.set_sensitive(False)
			squashfs_button.set_sensitive(False)

def changed_list_dists_combo4(widget):
	config_dist_button = xml.get_widget("config_dist_button")
	dist = widget.get_active_text()
	if dist == None:
		config_dist_button.set_sensitive(False)
	else:
		config_dist_button.set_sensitive(True)

def changed_dist_name_combo(widget):
	add_dist_button = xml.get_widget("add_dist_button")
	edit_dist_button = xml.get_widget("edit_dist_button")
	del_dist_button = xml.get_widget("del_dist_button")

	if widget.get_active_text() == "":
		add_dist_button.set_sensitive(False)
	else:
		add_dist_button.set_sensitive(True)


	model = widget.get_model()
	exists = False
	for model_unit in model:
		if model.get_value(model_unit.iter,0) == widget.get_active_text():
			exists = True

	if exists == True:
		edit_dist_button.set_sensitive(True)
		del_dist_button.set_sensitive(True)
		add_dist_button.set_sensitive(False)
	else:
		edit_dist_button.set_sensitive(False)
		del_dist_button.set_sensitive(False)


def changed_packages_combo(widget):
	add_package_button = xml.get_widget("add_package_button")
	del_package_button = xml.get_widget("del_package_button")

	if widget.get_active_text() == "":
		add_package_button.set_sensitive(False)
		del_package_button.set_sensitive(False)
	else:
		add_package_button.set_sensitive(True)

		model = widget.get_model()
		exists = False
		for model_unit in model:
			if model.get_value(model_unit.iter,0) == widget.get_active_text():
				exists = True
	
		if exists == True:
			del_package_button.set_sensitive(True)
			add_package_button.set_sensitive(False)
		else:
			del_package_button.set_sensitive(False)
			add_package_button.set_sensitive(True)


def changed_dns_servers_combo(widget):
	add_dns_server_button = xml.get_widget("add_dns_server_button")
	del_dns_server_button = xml.get_widget("del_dns_server_button")

	if widget.get_active_text() == "":
		add_dns_server_button.set_sensitive(False)
		del_dns_server_button.set_sensitive(False)
	else:
		add_dns_server_button.set_sensitive(True)

		model = widget.get_model()
		exists = False
		for model_unit in model:
			if model.get_value(model_unit.iter,0) == widget.get_active_text():
				exists = True

		if exists == True:
			del_dns_server_button.set_sensitive(True)
			add_dns_server_button.set_sensitive(False)
		else:
			del_dns_server_button.set_sensitive(False)
			add_dns_server_button.set_sensitive(True)


def changed_repositories_combo(widget):
	add_repository_button = xml.get_widget("add_repository_button")
	del_repository_button = xml.get_widget("del_repository_button")

	if widget.get_active_text() == "":
		add_repository_button.set_sensitive(False)
		del_repository_button.set_sensitive(False)
	else:
		add_repository_button.set_sensitive(True)

		model = widget.get_model()
		exists = False
		for model_unit in model:
			if model.get_value(model_unit.iter,0) == widget.get_active_text():
				exists = True
	
		if exists == True:
			del_repository_button.set_sensitive(True)
			add_repository_button.set_sensitive(False)
		else:
			del_repository_button.set_sensitive(False)
			add_repository_button.set_sensitive(True)

def clicked_del_dist(widget):
	dist_name_combo = xml.get_widget("dist_name_combo")
	shutil.rmtree("dists/"+dist_name_combo.get_active_text())
	dist_name_combo.set_active(0)
	model = dist_name_combo.get_model()
	model.remove(model[dist_name_combo.get_active()].iter)

def clicked_save_gendist_config(widget):
	write_gendist_config()

def clicked_cancel_gendist_config(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_current_page(0)

def clicked_add_dns_server(widget):
	dns_servers_combo = xml.get_widget("dns_servers_combo")
	dns_servers_model = dns_servers_combo.get_model()
	dns_servers_iter = dns_servers_model.insert_before(None)
	dns_servers_model.set_value(dns_servers_iter,0,dns_servers_combo.get_active_text())
	dns_servers_combo.get_child().set_text("")

def clicked_del_dns_server(widget):
	dns_servers_combo = xml.get_widget("dns_servers_combo")
	model = dns_servers_combo.get_model()
	iter = dns_servers_combo.get_active_iter()
	model.remove(iter)
	dns_servers_combo.get_child().set_text("")

def changed_dns_server_combo(widget):
	dns_servers_combo = xml.get_widget("dns_servers_combo")
	dns_servers_del_button = xml.get_widget("dns_servers_del_button")
	dns_servers_add_button = xml.get_widget("dns_servers_add_button")
	if dns_servers_combo.get_active_iter():
		dns_servers_del_button.set_sensitive(True)
		dns_servers_add_button.set_sensitive(False)
	else:
		dns_servers_del_button.set_sensitive(False)
		dns_servers_add_button.set_sensitive(True)

	if dns_servers_combo.get_active_text() == "":
		dns_servers_del_button.set_sensitive(False)
		dns_servers_add_button.set_sensitive(False)

	

def clicked_add_repository(widget):
	repositories_combo = xml.get_widget("repositories_combo")
	repositories_model = repositories_combo.get_model()
	repositories_iter = repositories_model.insert_before(None)
	repositories_model.set_value(repositories_iter,0,repositories_combo.get_active_text())
	repositories_combo.get_child().set_text("")

def clicked_del_repository(widget):
	repositories_combo = xml.get_widget("repositories_combo")
	model = repositories_combo.get_model()
	iter = repositories_combo.get_active_iter()
	model.remove(iter)
	repositories_combo.get_child().set_text("")

def clicked_add_package(widget):
	packages_combo = xml.get_widget("packages_combo")
	packages_model = packages_combo.get_model()
	packages_iter = packages_model.insert_before(None)
	packages_model.set_value(packages_iter,0,packages_combo.get_active_text())
	packages_combo.get_child().set_text("")

def clicked_del_package(widget):
	packages_combo = xml.get_widget("packages_combo")
	model = packages_combo.get_model()
	iter = packages_combo.get_active_iter()
	model.remove(iter)
	packages_combo.get_child().set_text("")

def clicked_splash_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	splash_open = xml.get_widget("splash_open_file")
	splash_open.show()

def clicked_splash_open_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	splash_open = xml.get_widget("splash_open_file")
	splash_input = xml.get_widget("splash_logo_input")
	splash_input.set_text(splash_open.get_filename())
	splash_open.hide()

def clicked_splash_open_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	splash_open = xml.get_widget("splash_open_file")
	splash_open.hide()

def clicked_doc_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(False)
	doc_open = xml.get_widget("doc_open_folder")
	doc_open.show()

def clicked_doc_open_open(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	doc_open = xml.get_widget("doc_open_folder")
	doc_input = xml.get_widget("doc_directory_input")
	doc_input.set_text(doc_open.get_filename())
	doc_open.hide()

def clicked_doc_open_cancel(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_sensitive(True)
	doc_open = xml.get_widget("doc_open_folder")
	doc_open.hide()

def clicked_save_dist_config(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_current_page(0)
	dist_name_combo = xml.get_widget("dist_name_combo")
	write_dist_config(dist_name_combo.get_active_text())

def clicked_cancel_dist_config(widget):
	main_notebook = xml.get_widget("main_notebook")
	main_notebook.set_current_page(0)

def clicked_cancel_create_dir(widget):
	create_folder_dialog = xml.get_widget("create_folder_dialog")
	create_folder_dialog.hide()

def create_kernels_model():
	kernels_model = gtk.ListStore( gobject.TYPE_STRING )
	if os.path.exists("/lib/modules"):
		kernels = os.listdir("/lib/modules")
		kernels.sort()
		for kernel in kernels:
			kernels_iter = kernels_model.insert_before(None)
			kernels_model.set(kernels_iter,0,kernel)
	return kernels_model

def create_modules_model():
	modules_model = gtk.ListStore( gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING )
	if os.path.exists("/usr/share/gendist/initramfs_modules"):
		modules = []
		for element in os.listdir("/usr/share/gendist/initramfs_modules"):
			if os.path.isfile("/usr/share/gendist/initramfs_modules/"+element):
				modules.append(element)
		modules.sort()
		for module in modules:
			modules_iter = modules_model.insert_before(None)
			modules_model.set(modules_iter,0,module)
	return modules_model

def get_module_parms(mod_name):
	mod_file =  file("/usr/share/gendist/initramfs_modules/"+mod_name)

	module_parms = []
	for line in mod_file:
		parms_match = re.match("# PARAMETERS=(.*)",line)
		if parms_match:
			parms_string = line.split("=")[1].replace("\n","")
			module_parm = []
			parm_type = parms_string.split("(")[0]
			module_parm.append(parm_type)
			parm_name = parms_string.replace(parm_type+"(","").split(",")[0].replace(")","")
			module_parm.append(parm_name)
			if parm_type == "ENUM":
				parm_values = parms_string.replace(parm_type+"("+parm_name+",","").replace(")","").split(",")
				module_parm.append(parm_values)
			module_parms.append(module_parm)
	return module_parms
		
def initialize_widgets():
	kernel_version_combo = xml.get_widget("kernel_version_combo")
	repositories_combo = xml.get_widget("repositories_combo")
	packages_combo = xml.get_widget("packages_combo")
	modules_treeview = xml.get_widget("modules_treeview")
	dns_servers_combo = xml.get_widget("dns_servers_combo")
	execute_frame = xml.get_widget("execute_frame")
	preferences_frame = xml.get_widget("preferences_frame")

	list_dists_combo1 = xml.get_widget("list_dists_combo1")
	list_dists_combo2 = xml.get_widget("list_dists_combo2")
	list_dists_combo3 = xml.get_widget("list_dists_combo3")
	list_dists_combo4 = xml.get_widget("list_dists_combo4")

	execute_frame.hide()
	preferences_frame.hide()

	renderer = gtk.CellRendererText()
	list_dists_combo1.pack_start(renderer,True)
	list_dists_combo1.add_attribute(renderer, 'text', 0)

	renderer = gtk.CellRendererText()
	list_dists_combo2.pack_start(renderer,True)
	list_dists_combo2.add_attribute(renderer, 'text', 0)

	renderer = gtk.CellRendererText()
	list_dists_combo3.pack_start(renderer,True)
	list_dists_combo3.add_attribute(renderer, 'text', 0)

	renderer = gtk.CellRendererText()
	list_dists_combo4.pack_start(renderer,True)
	list_dists_combo4.add_attribute(renderer, 'text', 0)

	renderer = gtk.CellRendererText()
	kernel_version_combo.pack_start(renderer, True)
	kernel_version_combo.add_attribute(renderer, 'text', 0)

	renderer = gtk.CellRendererText()
	column = gtk.TreeViewColumn("Name", renderer, text=0, background=2)
	modules_treeview.append_column(column)
	column = gtk.TreeViewColumn("Parameters", renderer, text=1, background=2)
	modules_treeview.append_column(column)

	modules_treeview.set_model(modules_model)

	vbox_terminal = xml.get_widget("vbox_terminal")
	vbox_terminal.add(terminal)

if __name__=='__main__':
	if os.getuid() > 0:
		print "Error: you must be root to run this software."
		sys.exit(1)

	# imitialize terminal
	terminal = vte.Terminal()
	terminal.show()
	terminal.connect("child-exited",child_exited_vte)

	xml = gtk.glade.XML("/usr/share/garban/garban.glade")
	xml.signal_autoconnect(locals())

	kernels_model = create_kernels_model()
	modules_model = create_modules_model()
	initialize_widgets()

	gtk.main()
